#!/usr/bin/env perl
#
# NAME:
#	build_install_list.pl - build and install_file.list
#
# SYNOPSIS:
#	find . -ls | build_install_list.pl
#
# DESCRIPTION:
#	This script turns the output of find -ls into the format
#	needed for install_files(8).
#	
# AUTHOR:
#	Simon J. Gerraty <sjg@crufty.net>
#

# RCSid:
#	$Id: build_install_list.pl,v 1.7 1996/07/31 10:24:37 sjg Exp $
#
#	@(#) Copyright (c) 1995-2001 Simon J. Gerraty
#
#	This file is provided in the hope that it will
#	be of use.  There is absolutely NO WARRANTY.
#	Permission to copy, redistribute or otherwise
#	use this file is hereby granted provided that 
#	the above copyright notice and this notice are
#	left intact. 
#      
#	Please send copies of changes and bug-fixes to:
#	sjg@crufty.net
#

eval 'exec /usr/local/bin/perl -S $0 ${1+"$@"}'
  if $running_under_some_shell;

require 'getopts.pl';
require 'basename.pl';

&Getopts('Shsp:');

%Types = ('d', 'directory',
	  'l', 'symlink',
	  );
%Perms = ('rwx', 7,
	  'rw-', 6,
	  'r-x', 5,
	  'r--', 4,
	  '-wx', 3,
	  '-w-', 2,
	  '--x', 1,
	  '---', 0,
	  );

$Ignore='/(RCS|CVS)/|(/RCS|/CVS|~|\.old|\.bak)$'; #'

%Inums = ();

# input should be from find -ls
while (<>) {
  chop;

  s/^\s+//;
  
  ($inum,$blocks,$mode,$nlinks,$owner,$group,$bytes,$file) = split(/\s+/, $_, 8);

  $file =~ s,^[^/]+,,;
  ($file,$val) = split(/\s+->\s+/, $file);

  next if ($file eq '');
  next if ($file =~ m/$Ignore/);
  $file = "$opt_p$file";
  
  ($type,$up,$gp,$op) = unpack("AA3A3A3", $mode);

  if (($type = $Types{$type}) eq '') {
    $type = 'file';
  }
  $action = 'no';		# for compatibility

  if ($type eq 'file' && $nlinks > 1) {
    if (($first = $Inums{$inum}) eq '') {
      $Inums{$inum} = "$file";
    } else {
      if (&dirname("$file") eq &dirname($first)) {
	$val = &basename($first);
      } else {
	$val = $first;
      }
      if ($opt_h ne '' || $opt_s eq '') {
	$type = 'hardlink';
	$action = "-> $val";
      } elsif ($opt_s ne '') {
	$type = 'symlink';
      }
    } 
  }
  if ($type eq 'symlink') {
    $mode = "0775";

    if ($val ne '' && $opt_s ne '') {
      $action = "-> $val";
    }

  } else {
    $sbits = 0;
    if ($up =~ m/s/) {
      $sbits |= 4;
      $up =~ s/s/x/;
    }
    if ($gp =~ m/s/) {
      $sbits |= 2;
      $gp =~ s/s/x/;
    }
    if ($gp =~ m/S/) {
      $sbits |= 2;
      $gp =~ s/s/-/;
    }
    if ($op =~ m/t/) {
      $sbits |= 1;
      $op =~ s/t/x/;
    }
    $mode = sprintf("%o$Perms{$up}$Perms{$gp}$Perms{$op}", $sbits);
  }

  if ($opt_S eq '') {
    printf "$file 0$mode $owner $group $type $action\n";
  } else {
    # building a remove_sbits.list
    $bits = '';
    $bits .= 'u' if ($sbits & 4);
    $bits .= 'g' if ($sbits & 2);
    $bits .= 't' if ($sbits & 1);

    printf "$file $bits\n";
  }
}
